package fr.jelora;

import java.util.Arrays;

import jssc.SerialPort;

public class TestCarteDebugProgZ80 {

	public static void main(String[] args) {
		TestCarteDebugProgZ80 testCarteDebugProgZ80 = new TestCarteDebugProgZ80();
		testCarteDebugProgZ80.run();
	}
	
	public void run(){
		try{
			byte[] programmeTest = new byte[]{
					// LD A,(1)
					(byte) 0b00111110,
					(byte) 0b00000001,
				// Debut
					// OUT (0),A
					(byte) 0b11010011,
					(byte) 0b00000000,
					// RLCA
					(byte) 0b00000111,
					// BIT (7),A
					(byte) 0b11001011,
					(byte) 0b01111111,
					// JR NZ,(3)
					(byte) 0b00100000,
					(byte) 0b00000011,
					// JP Debut
					(byte) 0b11000011,
					(byte) 0b00000010,
					(byte) 0b00000000,
				// Debut2
					// OUT (0),A
					(byte) 0b11010011,
					(byte) 0b00000000,
					// RRCA
					(byte) 0b00001111,
					// BIT (0),A
					(byte) 0b11001011,
					(byte) 0b01000111,
					// JRN Z,(3)
					(byte) 0b00100000,
					(byte) 0b00000011,
					// JP Debut2
					(byte) 0b11000011,
					(byte) 0b00001100,
					(byte) 0b00000000,
					// JP Debut
					(byte) 0b11000011,
					(byte) 0b00000010,
					(byte) 0b00000000
			};
			
			SerialPort serial = new SerialPort("COM3");
			serial.openPort();
			serial.setParams(SerialPort.BAUDRATE_19200, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			
			byte octet;
			byte[] commandeLECT = new byte[4];
			byte busDataValue;
			short busAdressValue;
			short busControlValue;
			BusControl busControl = new BusControl();
			boolean precedementEnROM = true;
			while ( true ){
				octet = serial.readBytes(1)[0];
				
				commandeLECT[0] = commandeLECT[1];
				commandeLECT[1] = commandeLECT[2];
				commandeLECT[2] = commandeLECT[3];
				commandeLECT[3] = octet;
				
				if ( Arrays.equals(commandeLECT, new byte[]{'L', 'E', 'C', 'T'}) ){
					
					busDataValue = serial.readBytes(1)[0];
					busControlValue = serial.readBytes(1)[0];
					busControlValue += serial.readBytes(1)[0] * 256;
					busAdressValue = serial.readBytes(1)[0];
					busAdressValue += serial.readBytes(1)[0] * 256;
					
					System.out.println("Control: " + byteToBitString(busControlValue));
					System.out.println("Adress:  " + byteToBitString(busAdressValue));
					System.out.println("Data:    " + byteToBitString(busDataValue));
					
					updateBusControl(busControl, busControlValue);
					if ( !busControl.MREQ && !busControl.RD ){
						System.out.print("ROM:     ");
						if ( busAdressValue >= 0 && busAdressValue <= 24 ){
							System.out.println(byteToBitString(programmeTest[busAdressValue]));
							
							serial.writeString("WED");
							serial.writeByte(programmeTest[busAdressValue]);
							precedementEnROM = true;
						}
						else{
							if ( precedementEnROM ){
								serial.writeString("DDA");
								precedementEnROM = false;
							}
						}
					}
					else{
						if ( precedementEnROM ){
							serial.writeString("DDA");
							precedementEnROM = false;
						}
					}
					
					
					
					System.out.println();
				}
				//System.out.println(new String(commandeLECT, "UTF-8"));
				//System.out.print(new String(new byte[]{octet}, "UTF-8"));
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	private void updateBusControl(BusControl busControl, short busControlValue){
		busControl.M1 = (busControlValue & 0b00000000000001) > 0;
		busControl.MREQ = (busControlValue & 0b00000000000010) > 0;
		busControl.IORQ = (busControlValue & 0b00000000000100) > 0;
		busControl.RD = (busControlValue & 0b00000000001000) > 0;
		busControl.WR = (busControlValue & 0b00000000010000) > 0;
		busControl.RFSH = (busControlValue & 0b00000000100000) > 0;
		busControl.HALT = (busControlValue & 0b00000001000000) > 0;
		busControl.WAIT = (busControlValue & 0b00000010000000) > 0;
		busControl.INT = (busControlValue & 0b00000100000000) > 0;
		busControl.NMI = (busControlValue & 0b00001000000000) > 0;
		busControl.RESET = (busControlValue & 0b00010000000000) > 0;
		busControl.BUSRQ = (busControlValue & 0b00100000000000) > 0;
		busControl.BUSAK = (busControlValue & 0b01000000000000) > 0;
		busControl.clock = (busControlValue & 0b10000000000000) > 0;
	}
	
	private class BusControl {
		// System control
		boolean M1;
		boolean MREQ;
		boolean IORQ;
		boolean RD;
		boolean WR;
		boolean RFSH;
		// CPU control
		boolean HALT;
		boolean WAIT;
		boolean INT;
		boolean NMI;
		boolean RESET;
		// CPU bus control
		boolean BUSRQ;
		boolean BUSAK;
		// Clock
		boolean clock;
		
	}
	
	private String byteToBitString(short valeur){
		StringBuilder bitString = new StringBuilder();
		
		bitString.append(byteToBitString((byte)(valeur / 256)));
		bitString.append(byteToBitString((byte)(valeur - ((valeur / 256)*256))));
		
		return bitString.toString();
	}
	
	private String byteToBitString(byte valeur){
		StringBuilder bitString = new StringBuilder();
		if ( (valeur & 0b10000000) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b01000000) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00100000) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00010000) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00001000) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00000100) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00000010) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		if ( (valeur & 0b00000001) > 0 ){
			bitString.append("1");
		}
		else{
			bitString.append("0");
		}
		return bitString.toString();
	}
}
